@@ -88,4 +88,7 @@ void                     ephy_embed_shell_register_ucm (EphyEmbedShell
 void                     ephy_embed_shell_unregister_ucm (EphyEmbedShell           *shell,
                                                           WebKitUserContentManager *ucm);
 
+void                     ephy_embed_shell_set_web_extension_initialization_data (EphyEmbedShell *shell,
+                                                                                 GVariant       *data);
+
 G_END_DECLS
