@@ -226,7 +226,7 @@ ephy_web_application_delete (const char      *id,
   /* If there's no profile dir for this app, it means it does not
    * exist. */
   if (!g_file_test (profile_dir, G_FILE_TEST_IS_DIR)) {
-    g_warning ("No application with id '%s' is installed.\n", id);
+    g_warning ("No application with id '%s' is installed.", id);
     if (out_app_found)
       *out_app_found = EPHY_WEB_APP_NOT_FOUND;
     return FALSE;
@@ -983,7 +983,7 @@ ephy_web_application_save (EphyWebApplication *app)
   g_assert (!ephy_is_running_inside_sandbox ());
 
   if (!g_file_get_contents (app->desktop_path, &contents, NULL, &error)) {
-    g_warning ("Failed to load desktop file of web application: %s\n", error->message);
+    g_warning ("Failed to load desktop file of web application: %s", error->message);
     return FALSE;
   }
 
@@ -1030,7 +1030,7 @@ ephy_web_application_save (EphyWebApplication *app)
 
     saved = g_key_file_save_to_file (keyfile, resolved_path, &error);
     if (!saved)
-      g_warning ("Failed to save web application %s desktop file %s: %s\n", app->name, resolved_path, error->message);
+      g_warning ("Failed to save web application %s desktop file %s: %s", app->name, resolved_path, error->message);
     free (resolved_path);
   }
 
