@@ -26,9 +26,12 @@
 #include "ephy-location-entry.h"
 
 #include "ephy-about-handler.h"
+#include "ephy-bookmark-properties.h"
 #include "ephy-debug.h"
+#include "ephy-embed-container.h"
 #include "ephy-embed-shell.h"
 #include "ephy-lib-type-builtins.h"
+#include "ephy-pixbuf-utils.h"
 #include "ephy-settings.h"
 #include "ephy-shell.h"
 #include "ephy-signal-accumulator.h"
@@ -59,6 +62,7 @@ struct _EphyLocationEntry {
   GtkWidget *text;
   GtkWidget *progress;
   GtkWidget *security_button;
+  GtkWidget *clear_button;
   GtkWidget *password_button;
   GtkWidget *bookmark_button;
   GtkWidget *reader_mode_button;
@@ -81,6 +85,8 @@ struct _EphyLocationEntry {
   gboolean reader_mode_active;
   gboolean show_suggestions;
 
+  gboolean select_all_selected;
+
   guint dns_prefetch_handle_id;
 
   guint user_changed : 1;
@@ -232,7 +238,11 @@ update_selected_url (EphyLocationEntry *entry)
     entry->jump_tab = g_strdup (uri);
     gtk_editable_set_text (GTK_EDITABLE (entry), dzl_suggestion_get_subtitle (suggestion));
   } else {
-    gtk_editable_set_text (GTK_EDITABLE (entry), uri);
+    if (ephy_suggestion_is_completion (EPHY_SUGGESTION (suggestion))) {
+      gtk_editable_set_text (GTK_EDITABLE (entry), ephy_suggestion_get_unescaped_title (EPHY_SUGGESTION (suggestion)));
+    } else {
+      gtk_editable_set_text (GTK_EDITABLE (entry), uri);
+    }
   }
   gtk_editable_set_position (GTK_EDITABLE (entry), -1);
   g_signal_handlers_unblock_by_func (entry, G_CALLBACK (editable_changed_cb), entry);
@@ -270,7 +280,7 @@ update_actions (EphyLocationEntry *entry)
 
   gtk_widget_action_set_enabled (entry->text, "selection.delete",
                                  editable && has_selection);
-  gtk_widget_action_set_enabled (entry->text, "selection.select-all",
+  gtk_widget_action_set_enabled (entry->text, "entry.select-all",
                                  has_content);
 
   gtk_widget_action_set_enabled (GTK_WIDGET (entry), "clipboard.paste-and-go",
@@ -293,15 +303,18 @@ update_suggestions_popover (EphyLocationEntry *entry)
   if (entry->show_suggestions && n_items > 0) {
     if (entry->adaptive_mode == EPHY_ADAPTIVE_MODE_NARROW) {
       GtkRoot *root = gtk_widget_get_root (GTK_WIDGET (entry));
-      double offset;
+      graphene_point_t entry_origin = GRAPHENE_POINT_INIT_ZERO;
+      graphene_point_t root_position;
+      gboolean ret;
 
-      gtk_widget_translate_coordinates (GTK_WIDGET (entry),
-                                        GTK_WIDGET (root),
-                                        0, 0,
-                                        &offset, NULL);
+      ret = gtk_widget_compute_point (GTK_WIDGET (entry),
+                                      GTK_WIDGET (root),
+                                      &entry_origin,
+                                      &root_position);
+      g_assert (ret);
 
       gtk_widget_set_halign (entry->suggestions_popover, GTK_ALIGN_START);
-      gtk_popover_set_offset (GTK_POPOVER (entry->suggestions_popover), -offset, 0);
+      gtk_popover_set_offset (GTK_POPOVER (entry->suggestions_popover), -root_position.x, 0);
     } else {
       gtk_widget_set_halign (entry->suggestions_popover, GTK_ALIGN_FILL);
       gtk_popover_set_offset (GTK_POPOVER (entry->suggestions_popover), 0, 0);
@@ -466,7 +479,7 @@ key_pressed_cb (EphyLocationEntry     *entry,
     if (selected == GTK_INVALID_LIST_POSITION)
       selected = matches - 1;
     else if (selected == 0)
-      selected = GTK_INVALID_LIST_POSITION;
+      selected = matches - 1;
     else
       selected--;
   }
@@ -475,7 +488,7 @@ key_pressed_cb (EphyLocationEntry     *entry,
     if (selected == GTK_INVALID_LIST_POSITION)
       selected = 0;
     else if (selected == matches - 1)
-      selected = GTK_INVALID_LIST_POSITION;
+      selected = 0;
     else
       selected++;
   }
@@ -551,7 +564,7 @@ get_suggestion_icon (GtkListItem *item,
     width = cairo_image_surface_get_width (surface);
     height = cairo_image_surface_get_height (surface);
 
-    return G_ICON (gdk_pixbuf_get_from_surface (surface, 0, 0, width, height));
+    return G_ICON (ephy_get_pixbuf_from_surface (surface, 0, 0, width, height));
   }
 
   if (icon)
@@ -579,7 +592,7 @@ get_suggestion_secondary_icon (GtkListItem *item,
     width = cairo_image_surface_get_width (surface);
     height = cairo_image_surface_get_height (surface);
 
-    return G_ICON (gdk_pixbuf_get_from_surface (surface, 0, 0, width, height));
+    return G_ICON (ephy_get_pixbuf_from_surface (surface, 0, 0, width, height));
   }
 
   if (icon)
@@ -601,6 +614,8 @@ update_entry_style (EphyLocationEntry *self,
   const char *base_domain;
   char *sub_string;
 
+  gtk_widget_set_visible (self->clear_button, focus);
+
   attrs = pango_attr_list_new ();
 
   if (focus)
@@ -646,6 +661,11 @@ focus_enter_cb (EphyLocationEntry *entry)
 static void
 focus_leave_cb (EphyLocationEntry *entry)
 {
+  if (entry->select_all_selected) {
+    entry->select_all_selected = FALSE;
+    return;
+  }
+
   update_entry_style (entry, FALSE);
   gtk_editable_select_region (GTK_EDITABLE (entry), 0, 0);
   set_show_suggestions (entry, FALSE);
@@ -708,9 +728,18 @@ static void
 editable_changed_cb (GtkEditable       *editable,
                      EphyLocationEntry *entry)
 {
+  GtkRoot *window = gtk_widget_get_root (GTK_WIDGET (entry));
+
   if (entry->block_update)
     return;
 
+  if (window) {
+    EphyEmbed *embed = ephy_window_get_active_embed (EPHY_WINDOW (window));
+    const char *text = gtk_editable_get_text (editable);
+
+    ephy_embed_set_typed_input (embed, text);
+  }
+
   entry->user_changed = TRUE;
   entry->can_redo = FALSE;
   update_actions (entry);
@@ -731,6 +760,13 @@ reader_mode_clicked_cb (EphyLocationEntry *entry)
                  entry->reader_mode_active);
 }
 
+static void
+clear_button_clicked_cb (EphyLocationEntry *self)
+{
+  gtk_editable_set_text (GTK_EDITABLE (self), "");
+  gtk_widget_grab_focus (GTK_WIDGET (self->text));
+}
+
 static void
 suggestions_popover_notify_visible_cb (EphyLocationEntry *entry)
 {
@@ -914,6 +950,13 @@ clear_activate (EphyLocationEntry *entry)
   update_actions (entry);
 }
 
+static void
+select_all_activate (EphyLocationEntry *entry)
+{
+  entry->select_all_selected = TRUE;
+  gtk_editable_select_region (GTK_EDITABLE (entry), 0, -1);
+}
+
 static void
 menu_popup_activate (EphyLocationEntry *entry)
 {
@@ -998,6 +1041,13 @@ ephy_location_entry_measure (GtkWidget      *widget,
       nat += child_nat;
     }
 
+    if (gtk_widget_should_layout (entry->clear_button)) {
+      gtk_widget_measure (entry->clear_button, orientation, for_size,
+                          &child_min, &child_nat, NULL, NULL);
+      min += child_min;
+      nat += child_nat;
+    }
+
     /* Since progress bar spans the whole width, we MAX() it instead of adding */
     gtk_widget_measure (entry->progress, orientation, for_size,
                         &child_min, &child_nat, NULL, NULL);
@@ -1079,6 +1129,8 @@ ephy_location_entry_size_allocate (GtkWidget *widget,
                  GTK_PACK_END, &icon_left_pos, &icon_right_pos);
   allocate_icon (widget, height, baseline, entry->reader_mode_button,
                  GTK_PACK_END, &icon_left_pos, &icon_right_pos);
+  allocate_icon (widget, height, baseline, entry->clear_button,
+                 GTK_PACK_END, &icon_left_pos, &icon_right_pos);
 
   for (l = entry->page_actions; l; l = l->next) {
     allocate_icon (widget, height, baseline, l->data,
@@ -1096,7 +1148,7 @@ ephy_location_entry_size_allocate (GtkWidget *widget,
                                  gtk_widget_get_width (root), -1);
   else
     gtk_widget_set_size_request (entry->suggestions_popover,
-                                 gtk_widget_get_allocated_width (widget), -1);
+                                 gtk_widget_get_width (widget), -1);
 
   gtk_widget_queue_resize (entry->suggestions_popover);
 
@@ -1254,6 +1306,7 @@ ephy_location_entry_dispose (GObject *object)
   gtk_widget_unparent (entry->password_button);
   gtk_widget_unparent (entry->bookmark_button);
   gtk_widget_unparent (entry->reader_mode_button);
+  gtk_widget_unparent (entry->clear_button);
   gtk_widget_unparent (entry->suggestions_popover);
 
   G_OBJECT_CLASS (ephy_location_entry_parent_class)->dispose (object);
@@ -1285,6 +1338,17 @@ register_activate_shortcuts (GtkWidgetClass  *widget_class,
                                 "i", modifiers);
 }
 
+static void
+on_bookmark_button_clicked (GtkButton *button,
+                            gpointer   user_data)
+{
+  GtkWidget *window = GTK_WIDGET (gtk_widget_get_root (GTK_WIDGET (button)));
+  GtkWidget *dialog;
+
+  dialog = ephy_bookmark_properties_new_for_window (EPHY_WINDOW (window));
+  adw_dialog_present (ADW_DIALOG (dialog), window);
+}
+
 static void
 ephy_location_entry_class_init (EphyLocationEntryClass *klass)
 {
@@ -1410,6 +1474,7 @@ ephy_location_entry_class_init (EphyLocationEntryClass *klass)
   gtk_widget_class_bind_template_child (widget_class, EphyLocationEntry, password_button);
   gtk_widget_class_bind_template_child (widget_class, EphyLocationEntry, bookmark_button);
   gtk_widget_class_bind_template_child (widget_class, EphyLocationEntry, reader_mode_button);
+  gtk_widget_class_bind_template_child (widget_class, EphyLocationEntry, clear_button);
   gtk_widget_class_bind_template_child (widget_class, EphyLocationEntry, suggestions_popover);
   gtk_widget_class_bind_template_child (widget_class, EphyLocationEntry, scrolled_window);
   gtk_widget_class_bind_template_child (widget_class, EphyLocationEntry, suggestions_model);
@@ -1436,6 +1501,7 @@ ephy_location_entry_class_init (EphyLocationEntryClass *klass)
   gtk_widget_class_bind_template_callback (widget_class, item_released_cb);
   gtk_widget_class_bind_template_callback (widget_class, get_suggestion_icon);
   gtk_widget_class_bind_template_callback (widget_class, get_suggestion_secondary_icon);
+  gtk_widget_class_bind_template_callback (widget_class, clear_button_clicked_cb);
 
   gtk_widget_class_set_css_name (widget_class, "entry");
   gtk_widget_class_set_accessible_role (widget_class, GTK_ACCESSIBLE_ROLE_TEXT_BOX);
@@ -1448,6 +1514,8 @@ ephy_location_entry_class_init (EphyLocationEntryClass *klass)
                                    (GtkWidgetActionActivateFunc)ephy_location_entry_reset);
   gtk_widget_class_install_action (widget_class, "edit.redo-extra", NULL,
                                    (GtkWidgetActionActivateFunc)ephy_location_entry_undo_reset);
+  gtk_widget_class_install_action (widget_class, "entry.select-all", NULL,
+                                   (GtkWidgetActionActivateFunc)select_all_activate);
   gtk_widget_class_install_action (widget_class, "menu.popup-extra", NULL,
                                    (GtkWidgetActionActivateFunc)menu_popup_activate);
 
@@ -1473,6 +1541,7 @@ ephy_location_entry_init (EphyLocationEntry *entry)
   entry->saved_text = NULL;
   entry->page_actions = NULL;
   entry->adaptive_mode = EPHY_ADAPTIVE_MODE_NORMAL;
+  entry->select_all_selected = FALSE;
 
   gtk_widget_init_template (GTK_WIDGET (entry));
 
@@ -1487,6 +1556,8 @@ ephy_location_entry_init (EphyLocationEntry *entry)
                    "visible",
                    G_SETTINGS_BIND_GET | G_SETTINGS_BIND_INVERT_BOOLEAN);
 
+  g_signal_connect (entry->bookmark_button, "clicked", G_CALLBACK (on_bookmark_button_clicked), entry);
+
   update_reader_icon (entry);
   g_signal_connect_object (gtk_settings_get_default (), "notify::gtk-icon-theme-name",
                            G_CALLBACK (update_reader_icon), entry, G_CONNECT_SWAPPED);
@@ -1754,16 +1825,16 @@ ephy_location_entry_set_bookmark_icon_state (EphyLocationEntry     *self,
       break;
     case EPHY_BOOKMARK_ICON_EMPTY:
       gtk_widget_set_visible (self->bookmark_button, TRUE);
-      gtk_menu_button_set_icon_name (GTK_MENU_BUTTON (self->bookmark_button),
-                                     "ephy-non-starred-symbolic");
+      gtk_button_set_icon_name (GTK_BUTTON (self->bookmark_button),
+                                "ephy-non-starred-symbolic");
       gtk_widget_remove_css_class (self->bookmark_button, "starred");
       /* Translators: tooltip for the empty bookmark button */
       gtk_widget_set_tooltip_text (self->bookmark_button, _("Bookmark Page"));
       break;
     case EPHY_BOOKMARK_ICON_BOOKMARKED:
       gtk_widget_set_visible (self->bookmark_button, TRUE);
-      gtk_menu_button_set_icon_name (GTK_MENU_BUTTON (self->bookmark_button),
-                                     "ephy-starred-symbolic");
+      gtk_button_set_icon_name (GTK_BUTTON (self->bookmark_button),
+                                "ephy-starred-symbolic");
       gtk_widget_add_css_class (self->bookmark_button, "starred");
       /* Translators: tooltip for the bookmarked button */
       gtk_widget_set_tooltip_text (self->bookmark_button, _("Edit Bookmark"));
@@ -1888,23 +1959,6 @@ ephy_location_entry_show_password_popover (EphyLocationEntry *entry)
   gtk_menu_button_popup (GTK_MENU_BUTTON (entry->password_button));
 }
 
-void
-ephy_location_entry_set_add_bookmark_popover (EphyLocationEntry *entry,
-                                              GtkPopover        *popover)
-{
-  g_assert (EPHY_IS_LOCATION_ENTRY (entry));
-  g_assert (GTK_IS_POPOVER (popover));
-
-  gtk_menu_button_set_popover (GTK_MENU_BUTTON (entry->bookmark_button),
-                               GTK_WIDGET (popover));
-}
-
-void
-ephy_location_entry_show_add_bookmark_popover (EphyLocationEntry *entry)
-{
-  gtk_menu_button_popup (GTK_MENU_BUTTON (entry->bookmark_button));
-}
-
 void
 ephy_location_entry_set_reader_mode_visible (EphyLocationEntry *entry,
                                              gboolean           visible)
