@@ -19,6 +19,7 @@
 #include "config.h"
 #include "ephy-suggestion-model.h"
 
+#include "ephy-about-handler.h"
 #include "ephy-embed-shell.h"
 #include "ephy-prefs.h"
 #include "ephy-search-engine-manager.h"
@@ -27,7 +28,7 @@
 #include "ephy-user-agent.h"
 #include "ephy-window.h"
 
-#include <dazzle.h>
+#include "dzl-fuzzy-mutable-index.h"
 #include <glib/gi18n.h>
 
 #define MAX_URL_ENTRIES             25
@@ -59,8 +60,8 @@ enum {
 
 static void list_model_iface_init (GListModelInterface *iface);
 
-G_DEFINE_TYPE_WITH_CODE (EphySuggestionModel, ephy_suggestion_model, G_TYPE_OBJECT,
-                         G_IMPLEMENT_INTERFACE (G_TYPE_LIST_MODEL, list_model_iface_init))
+G_DEFINE_FINAL_TYPE_WITH_CODE (EphySuggestionModel, ephy_suggestion_model, G_TYPE_OBJECT,
+                               G_IMPLEMENT_INTERFACE (G_TYPE_LIST_MODEL, list_model_iface_init))
 
 static GParamSpec *properties[N_PROPS];
 
@@ -132,15 +133,13 @@ ephy_suggestion_model_class_init (EphySuggestionModelClass *klass)
 
   properties [PROP_BOOKMARKS_MANAGER] =
     g_param_spec_object ("bookmarks-manager",
-                         "Bookmarks Manager",
-                         "The bookmarks manager for suggestions",
+                         NULL, NULL,
                          EPHY_TYPE_BOOKMARKS_MANAGER,
                          (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   properties [PROP_HISTORY_SERVICE] =
     g_param_spec_object ("history-service",
-                         "History Service",
-                         "The history service for suggestions",
+                         NULL, NULL,
                          EPHY_TYPE_HISTORY_SERVICE,
                          (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
@@ -177,8 +176,10 @@ ephy_suggestion_model_get_item (GListModel *model,
   DzlSuggestion *suggestion;
 
   iter = g_sequence_get_iter_at_pos (self->items, position);
-  suggestion = g_sequence_get (iter);
+  if (g_sequence_iter_is_end (iter))
+    return NULL;
 
+  suggestion = g_sequence_get (iter);
   return g_object_ref (suggestion);
 }
 
@@ -254,23 +255,27 @@ icon_loaded_cb (GObject      *source,
 {
   WebKitFaviconDatabase *database = WEBKIT_FAVICON_DATABASE (source);
   EphySuggestion *suggestion;
-  GError *error = NULL;
+  g_autoptr (GdkTexture) texture = NULL;
   cairo_surface_t *favicon;
   gdouble x_scale, y_scale;
-  int x, y;
+  int w, h;
 
-  favicon = webkit_favicon_database_get_favicon_finish (database, result, &error);
-
-  if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED) || favicon == NULL)
+  texture = webkit_favicon_database_get_favicon_finish (database, result, NULL);
+  if (texture == NULL)
     return;
 
   suggestion = EPHY_SUGGESTION (user_data);
 
-  x = cairo_image_surface_get_width (favicon);
-  y = cairo_image_surface_get_height (favicon);
-  x_scale = (gdouble)x / 16;
-  y_scale = (gdouble)y / 16;
+  w = gdk_texture_get_width (texture);
+  h = gdk_texture_get_height (texture);
+  favicon = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, w, h);
+  gdk_texture_download (texture,
+                        cairo_image_surface_get_data (favicon),
+                        cairo_image_surface_get_stride (favicon));
+  cairo_surface_mark_dirty (favicon);
 
+  x_scale = (gdouble)w / 16;
+  y_scale = (gdouble)h / 16;
   cairo_surface_set_device_scale (favicon, x_scale, y_scale);
 
   ephy_suggestion_set_favicon (suggestion, favicon);
@@ -282,8 +287,7 @@ load_favicon (EphySuggestionModel *model,
               const char          *url)
 {
   EphyEmbedShell *shell = ephy_embed_shell_get_default ();
-  WebKitWebContext *context = ephy_embed_shell_get_web_context (shell);
-  WebKitFaviconDatabase *database = webkit_web_context_get_favicon_database (context);
+  WebKitFaviconDatabase *database = ephy_embed_shell_get_favicon_database (shell);
 
   webkit_favicon_database_get_favicon (database,
                                        url,
@@ -338,9 +342,9 @@ add_search_engines (EphySuggestionModel *self,
     address = ephy_search_engine_build_search_address (engine, query);
     escaped_title = g_markup_escape_text (engine_name, -1);
     markup = dzl_fuzzy_highlight (escaped_title, query, FALSE);
-    suggestion = ephy_suggestion_new (markup, engine_name, address);
+    suggestion = ephy_suggestion_new (markup, engine_name, address, FALSE);
 
-    uri = g_uri_parse (address, G_URI_FLAGS_NONE, NULL);
+    uri = g_uri_parse (address, G_URI_FLAGS_PARSE_RELAXED, NULL);
     if (uri) {
       g_free (address);
       address = g_strdup_printf ("%s://%s/", g_uri_get_scheme (uri), g_uri_get_host (uri));
@@ -542,6 +546,12 @@ tabs_query (EphySuggestionModel *self,
       if ((title_casefold && strstr (title_casefold, query_casefold)) || strstr (display_address_casefold, query_casefold)) {
         char *escaped_address = g_markup_escape_text (display_address, -1);
 
+        if (g_str_has_prefix (escaped_address, EPHY_ABOUT_SCHEME)) {
+          g_autofree char *pretty_address = g_strconcat ("about", escaped_address + EPHY_ABOUT_SCHEME_LEN, NULL);
+          g_free (escaped_address);
+          escaped_address = g_steal_pointer (&pretty_address);
+        }
+
         escaped_title = g_markup_escape_text (title, -1);
         markup = dzl_fuzzy_highlight (escaped_title, data->query, FALSE);
         suggestion = ephy_suggestion_new_with_custom_subtitle (markup, title, escaped_address, address);
@@ -560,7 +570,6 @@ bookmarks_query (EphySuggestionModel *self,
                  QueryData           *data,
                  GTask               *task)
 {
-  g_autoptr (GList) new_urls = NULL;
   GSequence *bookmarks;
 
   bookmarks = ephy_bookmarks_manager_get_bookmarks (self->bookmarks_manager);
@@ -584,10 +593,16 @@ bookmarks_query (EphySuggestionModel *self,
       EphySuggestion *suggestion;
       g_autofree gchar *escaped_title = NULL;
       g_autofree gchar *markup = NULL;
+      g_autofree gchar *pretty_url = NULL;
+
+      if (g_str_has_prefix (url, EPHY_ABOUT_SCHEME)) {
+        pretty_url = g_strconcat ("about", url + EPHY_ABOUT_SCHEME_LEN, NULL);
+        url = pretty_url;
+      }
 
       escaped_title = g_markup_escape_text (title, -1);
       markup = dzl_fuzzy_highlight (escaped_title, data->query, FALSE);
-      suggestion = ephy_suggestion_new (markup, title, url);
+      suggestion = ephy_suggestion_new (markup, title, url, FALSE);
       ephy_suggestion_set_secondary_icon (suggestion, "ephy-starred-symbolic");
 
       g_sequence_append (data->bookmarks, g_object_ref (suggestion));
@@ -626,7 +641,7 @@ history_query_completed_cb (EphyHistoryService *service,
       escaped_title = g_markup_escape_text (title, -1);
 
       markup = dzl_fuzzy_highlight (escaped_title, data->query, FALSE);
-      suggestion = ephy_suggestion_new (markup, title, url->url);
+      suggestion = ephy_suggestion_new (markup, title, url->url, FALSE);
 
       g_sequence_append (data->history, g_steal_pointer (&suggestion));
     }
@@ -635,17 +650,10 @@ history_query_completed_cb (EphyHistoryService *service,
   query_collection_done (self, g_steal_pointer (&task));
 }
 
-#if SOUP_CHECK_VERSION (2, 99, 4)
 static void
 google_search_suggestions_cb (SoupSession  *session,
                               GAsyncResult *result,
                               gpointer      user_data)
-#else
-static void
-google_search_suggestions_cb (SoupSession *session,
-                              SoupMessage *msg,
-                              gpointer     user_data)
-#endif
 {
   GTask *task = G_TASK (user_data);
   EphySuggestionModel *self = g_task_get_source_object (task);
@@ -658,7 +666,7 @@ google_search_suggestions_cb (SoupSession *session,
   EphySearchEngine *engine;
   int added = 0;
   g_autoptr (GBytes) body = NULL;
-#if SOUP_CHECK_VERSION (2, 99, 4)
+
   SoupMessage *msg;
 
   body = soup_session_send_and_read_finish (session, result, NULL);
@@ -670,12 +678,6 @@ google_search_suggestions_cb (SoupSession *session,
     goto out;
 
   /* FIXME: we don't have a way to get the status code */
-#else
-  if (msg->status_code != 200)
-    goto out;
-
-  body = g_bytes_new_static (msg->response_body->data, msg->response_body->length);
-#endif
 
   shell = ephy_embed_shell_get_default ();
   manager = ephy_embed_shell_get_search_engine_manager (shell);
@@ -697,13 +699,11 @@ google_search_suggestions_cb (SoupSession *session,
     g_autofree char *address = NULL;
     g_autofree char *escaped_title = NULL;
     g_autofree char *markup = NULL;
-    const char *engine_name;
 
     address = ephy_search_engine_build_search_address (engine, str);
     escaped_title = g_markup_escape_text (str, -1);
     markup = dzl_fuzzy_highlight (escaped_title, str, FALSE);
-    engine_name = ephy_search_engine_get_name (engine);
-    suggestion = ephy_suggestion_new (markup, engine_name, address);
+    suggestion = ephy_suggestion_new (markup, escaped_title, address, TRUE);
 
     g_sequence_append (data->google_suggestions, g_steal_pointer (&suggestion));
     added++;
@@ -731,14 +731,10 @@ google_search_suggestions_query (EphySuggestionModel *self,
   escaped_query = g_markup_escape_text (query, -1);
   url = g_strdup_printf ("http://suggestqueries.google.com/complete/search?client=firefox&q=%s", escaped_query);
   msg = soup_message_new (SOUP_METHOD_GET, url);
-#if SOUP_CHECK_VERSION (2, 99, 4)
   soup_session_send_and_read_async (self->session, msg, G_PRIORITY_DEFAULT, NULL,
                                     (GAsyncReadyCallback)google_search_suggestions_cb,
                                     g_steal_pointer (&task));
   g_object_unref (msg);
-#else
-  soup_session_queue_message (self->session, g_steal_pointer (&msg), google_search_suggestions_cb, g_steal_pointer (&task));
-#endif
 }
 
 void
