@@ -50,11 +50,15 @@
 #include "api/windows.h"
 
 #include <adwaita.h>
+#include <archive.h>
+#include <archive_entry.h>
 #include <json-glib/json-glib.h>
 
 static void handle_message_reply (EphyWebExtension *web_extension,
                                   JsonArray        *args);
 
+static char *get_translation_contents (EphyWebExtension *web_extension);
+
 struct _EphyWebExtensionManager {
   GObject parent_instance;
 
@@ -115,11 +119,48 @@ create_user_agent_overrides (void)
   return overrides;
 }
 
+static GVariant *
+create_extension_data_variant (EphyWebExtension *extension)
+{
+  g_auto (GVariantDict) dict = G_VARIANT_DICT_INIT (NULL);
+  g_autofree char *translations = get_translation_contents (extension);
+
+  g_variant_dict_insert (&dict, "manifest", "s", ephy_web_extension_get_manifest (extension));
+  g_variant_dict_insert (&dict, "translations", "s", translations);
+  g_variant_dict_insert (&dict, "has-background-page", "b", ephy_web_extension_has_background_web_view (extension));
+
+  return g_variant_dict_end (&dict);
+}
+
+static GVariant *
+ephy_web_extension_manager_get_extension_initialization_data (EphyWebExtensionManager *self)
+{
+  g_auto (GVariantDict) dict = G_VARIANT_DICT_INIT (NULL);
+
+  for (guint i = 0; i < self->web_extensions->len; i++) {
+    EphyWebExtension *extension = g_ptr_array_index (self->web_extensions, i);
+
+    g_variant_dict_insert_value (&dict,
+                                 ephy_web_extension_get_guid (extension),
+                                 create_extension_data_variant (extension));
+  }
+
+  return g_variant_dict_end (&dict);
+}
+
+static void
+ephy_web_extension_manager_update_extension_initialization_data (EphyWebExtensionManager *self)
+{
+  EphyEmbedShell *embed = ephy_embed_shell_get_default ();
+  ephy_embed_shell_set_web_extension_initialization_data (embed, ephy_web_extension_manager_get_extension_initialization_data (self));
+}
+
 static void
 ephy_web_extension_manager_add_to_list (EphyWebExtensionManager *self,
                                         EphyWebExtension        *web_extension)
 {
   g_ptr_array_add (self->web_extensions, g_object_ref (web_extension));
+  ephy_web_extension_manager_update_extension_initialization_data (self);
   g_signal_emit (self, signals[CHANGED], 0);
 }
 
@@ -128,6 +169,7 @@ ephy_web_extension_manager_remove_from_list (EphyWebExtensionManager *self,
                                              EphyWebExtension        *web_extension)
 {
   g_ptr_array_remove (self->web_extensions, web_extension);
+  ephy_web_extension_manager_update_extension_initialization_data (self);
   g_signal_emit (self, signals[CHANGED], 0);
 }
 
@@ -137,7 +179,7 @@ ephy_web_extension_manager_get_extension_by_guid (EphyWebExtensionManager *self,
 {
   for (guint i = 0; i < self->web_extensions->len; i++) {
     EphyWebExtension *web_extension = g_ptr_array_index (self->web_extensions, i);
-    if (strcmp (guid, ephy_web_extension_get_guid (web_extension)) == 0)
+    if (g_strcmp0 (guid, ephy_web_extension_get_guid (web_extension)) == 0)
       return web_extension;
   }
 
@@ -218,11 +260,15 @@ ephy_webextension_scheme_cb (WebKitURISchemeRequest *request,
                              gpointer                user_data)
 {
   EphyWebExtensionManager *self = ephy_web_extension_manager_get_default ();
-  EphyWebExtension *web_extension = user_data;
+  WebKitWebView *initiating_view;
+  EphyWebExtension *web_extension;
   EphyWebExtension *target_web_extension;
   g_autoptr (GInputStream) stream = NULL;
   g_autoptr (GUri) uri = NULL;
+  const char *initiating_uri_string;
+  g_autoptr (GUri) initiating_uri = NULL;
   g_autoptr (GError) error = NULL;
+  const char *initating_host;
   const unsigned char *data;
   gsize length;
 
@@ -241,6 +287,26 @@ ephy_webextension_scheme_cb (WebKitURISchemeRequest *request,
     return;
   }
 
+  /* Figure out the source of this request. */
+  initiating_view = webkit_uri_scheme_request_get_web_view (request);
+  if (EPHY_IS_WEB_VIEW (initiating_view))
+    initiating_uri_string = ephy_web_view_get_address (EPHY_WEB_VIEW (initiating_view));
+  else
+    initiating_uri_string = webkit_web_view_get_uri (initiating_view);
+  if (!initiating_uri_string) {
+    webkit_uri_scheme_request_finish_error (request, g_error_new (WEB_EXTENSION_ERROR, WEB_EXTENSION_ERROR_INVALID_HOST, _("Failed to determine initiating URI")));
+    return;
+  }
+
+  initiating_uri = g_uri_parse (initiating_uri_string, G_URI_FLAGS_NON_DNS, &error);
+  if (!initiating_uri) {
+    webkit_uri_scheme_request_finish_error (request, g_steal_pointer (&error));
+    return;
+  }
+
+  initating_host = g_uri_get_host (initiating_uri);
+  web_extension = ephy_web_extension_manager_get_extension_by_guid (self, initating_host);
+
   /* If this is not originating from the same WebExtension view we must find it and filter it by web_accessible_resources. */
   if (web_extension != target_web_extension) {
     if (!ephy_web_extension_has_web_accessible_resource (target_web_extension, g_uri_get_path (uri) + 1)) {
@@ -392,6 +458,163 @@ on_new_web_extension_loaded (GObject      *source_object,
   ephy_web_extension_manager_add_to_list (self, web_extension);
 }
 
+static char *
+decompress_xpi_finish (EphyWebExtensionManager  *self,
+                       GAsyncResult             *result,
+                       GError                  **error)
+{
+  return g_task_propagate_pointer (G_TASK (result), error);
+}
+
+static void
+on_extension_decompressed (GObject      *source,
+                           GAsyncResult *res,
+                           gpointer      user_data)
+{
+  EphyWebExtensionManager *self = EPHY_WEB_EXTENSION_MANAGER (user_data);
+  g_autoptr (GError) error = NULL;
+  g_autoptr (GFile) target = NULL;
+  GFileInfo *file_info;
+  g_autofree char *path = decompress_xpi_finish (self, res, &error);
+
+  if (error) {
+    if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+      g_warning ("Could not decompress WebExtension: %s", error->message);
+    return;
+  }
+
+  target = g_file_new_for_path (path);
+  file_info = g_file_query_info (target, G_FILE_ATTRIBUTE_STANDARD_TYPE, 0, NULL, &error);
+  if (!file_info) {
+    g_warning ("Failed to query file info: %s", error->message);
+    return;
+  }
+
+  ephy_web_extension_load_async (g_steal_pointer (&target), file_info, self->cancellable, on_new_web_extension_loaded, self);
+}
+
+static int
+copy_data (struct archive *ar,
+           struct archive *aw)
+{
+  int r;
+  const void *buff;
+  size_t size;
+  la_int64_t offset;
+
+  for (;;) {
+    r = archive_read_data_block (ar, &buff, &size, &offset);
+    if (r == ARCHIVE_EOF)
+      return (ARCHIVE_OK);
+    if (r < ARCHIVE_OK)
+      return (r);
+    r = archive_write_data_block (aw, buff, size, offset);
+    if (r < ARCHIVE_OK) {
+      g_warning ("Failed to copy archive data: %s", archive_error_string (aw));
+      return (r);
+    }
+  }
+}
+
+static void
+decompress_xpi_thread (GTask        *task,
+                       gpointer      source_object,
+                       gpointer      task_data,
+                       GCancellable *cancellable)
+{
+  GFile *file = G_FILE (source_object);
+  GFile *web_extensions_dir = task_data;
+  struct archive *archive;
+  struct archive *ext;
+  int flags;
+  int ret;
+  const char *filename = g_file_get_path (file);
+  g_autofree char *path = NULL;
+  g_autofree char *basename = NULL;
+
+  flags = ARCHIVE_EXTRACT_TIME;
+  flags |= ARCHIVE_EXTRACT_PERM;
+  flags |= ARCHIVE_EXTRACT_ACL;
+  flags |= ARCHIVE_EXTRACT_FFLAGS;
+
+  archive = archive_read_new ();
+  archive_read_support_format_all (archive);
+  archive_read_support_filter_all (archive);
+
+  ext = archive_write_disk_new ();
+  archive_write_disk_set_options (ext, flags);
+  archive_write_disk_set_standard_lookup (ext);
+
+  ret = archive_read_open_filename (archive, filename, 10240);
+  if (ret) {
+    g_warning ("Could not open archive: %s", filename);
+    return;
+  }
+
+  basename = g_file_get_basename (file);
+  path = g_build_filename (g_file_peek_path (web_extensions_dir), basename, NULL);
+
+  while (1) {
+    struct archive_entry *entry;
+    g_autofree char *full_path = NULL;
+
+    ret = archive_read_next_header (archive, &entry);
+    if (ret == ARCHIVE_EOF)
+      break;
+
+    if (ret < ARCHIVE_OK)
+      g_warning ("Error extracting archive: %s", archive_error_string (archive));
+
+    if (ret < ARCHIVE_WARN)
+      return;
+
+    full_path = g_build_filename (path, archive_entry_pathname (entry), NULL);
+    archive_entry_set_pathname (entry, full_path);
+    ret = archive_write_header (ext, entry);
+    if (ret < ARCHIVE_OK)
+      g_warning ("Could not write archive: %s", archive_error_string (ext));
+    else if (archive_entry_size (entry) > 0) {
+      ret = copy_data (archive, ext);
+      if (ret < ARCHIVE_OK)
+        g_warning ("Could not copy archive data: %s", archive_error_string (ext));
+      if (ret < ARCHIVE_WARN)
+        return;
+    }
+
+    ret = archive_write_finish_entry (ext);
+    if (ret < ARCHIVE_OK)
+      g_warning ("Could not finish archive: %s", archive_error_string (ext));
+    if (ret < ARCHIVE_WARN)
+      return;
+  }
+
+  archive_read_close (archive);
+  archive_read_free (archive);
+  archive_write_close (ext);
+  archive_write_free (ext);
+
+  g_task_return_pointer (task, g_steal_pointer (&path), g_free);
+}
+
+static void
+decompress_xpi (GFile               *extension,
+                GFile               *web_extensions_dir,
+                GCancellable        *cancellable,
+                GAsyncReadyCallback  callback,
+                gpointer             user_data)
+{
+  g_autoptr (GTask) task = NULL;
+
+  g_assert (extension);
+  g_assert (web_extensions_dir);
+
+  task = g_task_new (extension, cancellable, callback, user_data);
+  g_task_set_task_data (task, g_object_ref (web_extensions_dir), g_object_unref);
+  g_task_set_return_on_cancel (task, TRUE);
+
+  g_task_run_in_thread (task, decompress_xpi_thread);
+}
+
 /**
  * Install a new web web_extension into the local web_extension directory.
  * File should only point to a manifest.json or a .xpi file
@@ -401,11 +624,11 @@ ephy_web_extension_manager_install (EphyWebExtensionManager *self,
                                     GFile                   *file)
 {
   g_autoptr (GFile) target = NULL;
-  g_autofree char *basename = NULL;
   g_autoptr (GFileInfo) file_info = NULL;
   gboolean is_xpi = FALSE;
   g_autoptr (GError) error = NULL;
   g_autoptr (GFile) web_extensions_dir = NULL;
+  g_autofree char *basename = NULL;
   const char *path;
 
   web_extensions_dir = g_file_new_build_filename (ephy_default_profile_dir (), "web_extensions", NULL);
@@ -416,22 +639,7 @@ ephy_web_extension_manager_install (EphyWebExtensionManager *self,
   /* FIXME: Make this async. */
 
   if (is_xpi) {
-    /* If we are given an .xpi file its a direct copy. */
-    basename = g_file_get_basename (file);
-    target = g_file_get_child (web_extensions_dir, basename);
-
-    if (!g_file_make_directory_with_parents (web_extensions_dir, NULL, &error)) {
-      if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_EXISTS)) {
-        g_warning ("Failed to create web_extensions directory: %s", error->message);
-        return;
-      }
-      g_clear_error (&error);
-    }
-
-    if (!g_file_copy (file, target, G_FILE_COPY_NONE, NULL, NULL, NULL, &error)) {
-      g_warning ("Could not copy file for web_extension: %s", error->message);
-      return;
-    }
+    decompress_xpi (file, web_extensions_dir, self->cancellable, on_extension_decompressed, self);
   } else {
     /* Otherwise we copy the parent directory. */
     g_autoptr (GFile) parent = g_file_get_parent (file);
@@ -439,16 +647,16 @@ ephy_web_extension_manager_install (EphyWebExtensionManager *self,
     target = g_file_get_child (web_extensions_dir, basename);
 
     ephy_copy_directory (g_file_peek_path (parent), g_file_peek_path (target));
-  }
 
-  if (target) {
-    file_info = g_file_query_info (target, G_FILE_ATTRIBUTE_STANDARD_TYPE, 0, self->cancellable, &error);
-    if (!file_info) {
-      g_warning ("Failed to query file info: %s", error->message);
-      return;
+    if (target) {
+      file_info = g_file_query_info (target, G_FILE_ATTRIBUTE_STANDARD_TYPE, 0, self->cancellable, &error);
+      if (!file_info) {
+        g_warning ("Failed to query file info: %s", error->message);
+        return;
+      }
+
+      ephy_web_extension_load_async (g_steal_pointer (&target), file_info, self->cancellable, on_new_web_extension_loaded, self);
     }
-
-    ephy_web_extension_load_async (g_steal_pointer (&target), file_info, self->cancellable, on_new_web_extension_loaded, self);
   }
 }
 
@@ -856,17 +1064,7 @@ get_translation_contents (EphyWebExtension *web_extension)
   g_autofree char *path = g_strdup_printf ("_locales/%s/messages.json", "en");
   g_autofree char *data = ephy_web_extension_get_resource_as_string (web_extension, path);
 
-  return data ? g_steal_pointer (&data) : g_strdup ("");
-}
-
-static void
-update_translations (EphyWebExtension *web_extension)
-{
-  g_autofree char *data = get_translation_contents (web_extension);
-
-  webkit_web_context_send_message_to_all_extensions (ephy_embed_shell_get_web_context (ephy_embed_shell_get_default ()),
-                                                     webkit_user_message_new ("WebExtension.UpdateTranslations",
-                                                                              g_variant_new ("(ss)", ephy_web_extension_get_guid (web_extension), data)));
+  return data ? g_steal_pointer (&data) : g_strdup ("{}");
 }
 
 static void
@@ -877,7 +1075,6 @@ send_to_page_ready_cb (WebKitWebView    *web_view,
   g_autoptr (WebKitUserMessage) response = webkit_web_view_send_message_to_page_finish (web_view, result, NULL);
   (void)response;
 
-  update_translations (web_extension);
   add_content_scripts (web_extension, EPHY_WEB_VIEW (web_view));
 }
 
@@ -922,7 +1119,8 @@ ephy_web_extension_manager_add_web_extension_to_webview (EphyWebExtensionManager
                     web_extension);
 
   webkit_web_view_send_message_to_page (WEBKIT_WEB_VIEW (web_view),
-                                        webkit_user_message_new ("WebExtension.Initialize", g_variant_new_string (ephy_web_extension_get_guid (web_extension))),
+                                        webkit_user_message_new ("WebExtension.Initialize",
+                                                                 g_variant_new ("(sv)", ephy_web_extension_get_guid (web_extension), create_extension_data_variant (web_extension))),
                                         NULL, (GAsyncReadyCallback)send_to_page_ready_cb, web_extension);
 }
 
@@ -947,6 +1145,7 @@ init_web_extension_api (WebKitWebContext *web_context,
                         EphyWebExtension *web_extension)
 {
   g_autoptr (GVariant) user_data = NULL;
+  EphyWebExtensionManager *manager = ephy_web_extension_manager_get_default ();
   g_autofree char *translations = get_translation_contents (web_extension);
 
 #if DEVELOPER_MODE
@@ -955,13 +1154,12 @@ init_web_extension_api (WebKitWebContext *web_context,
   webkit_web_context_set_web_process_extensions_directory (web_context, EPHY_WEB_PROCESS_EXTENSIONS_DIR);
 #endif
 
-  user_data = g_variant_new ("(smsbbbs)",
+  user_data = g_variant_new ("(smsbbv)",
                              ephy_web_extension_get_guid (web_extension),
                              ephy_profile_dir_is_default () ? NULL : ephy_profile_dir (),
                              FALSE /* should_remember_passwords */,
                              FALSE /* private_profile */,
-                             TRUE /* is_webextension */,
-                             translations);
+                             ephy_web_extension_manager_get_extension_initialization_data (manager));
   webkit_web_context_set_web_process_extensions_initialization_user_data (web_context, g_steal_pointer (&user_data));
 }
 
@@ -1037,7 +1235,7 @@ ephy_web_extensions_manager_create_web_extensions_webview (EphyWebExtension *web
   background_view = ephy_web_extension_manager_get_background_web_view (manager, web_extension);
   if (!background_view) {
     web_context = webkit_web_context_new ();
-    webkit_web_context_register_uri_scheme (web_context, "ephy-webextension", ephy_webextension_scheme_cb, web_extension, NULL);
+    webkit_web_context_register_uri_scheme (web_context, "ephy-webextension", ephy_webextension_scheme_cb, NULL, NULL);
     webkit_security_manager_register_uri_scheme_as_secure (webkit_web_context_get_security_manager (web_context),
                                                            "ephy-webextension");
     g_signal_connect_object (web_context, "initialize-web-process-extensions", G_CALLBACK (init_web_extension_api), web_extension, 0);
